@@ -25,6 +25,7 @@ module Control.Monad.Trans.Resource
       -- * Unwrap
     , runResourceT
       -- * Special actions
+    , resourceForkWith
     , resourceForkIO
       -- * Monad transformation
     , transResourceT
@@ -131,8 +132,8 @@ resourceMask :: MonadResource m => ((forall a. ResourceT IO a -> ResourceT IO a)
 resourceMask r = liftResourceT (resourceMaskRIO r)
 
 allocateRIO :: IO a -> (a -> IO ()) -> ResourceT IO (ReleaseKey, a)
-allocateRIO acquire rel = ResourceT $ \istate -> liftIO $ E.mask $ \restore -> do
-    a <- restore acquire
+allocateRIO acquire rel = ResourceT $ \istate -> liftIO $ E.mask_ $ do
+    a <- acquire
     key <- register' istate $ rel a
     return (key, a)
 
@@ -243,17 +244,20 @@ runException_ = runIdentity . runExceptionT_
 -- shared by multiple threads. Once the last thread exits, all remaining
 -- resources will be released.
 --
+-- The first parameter is a function which will be used to create the
+-- thread, such as @forkIO@ or @async@.
+--
 -- Note that abuse of this function will greatly delay the deallocation of
 -- registered resources. This function should be used with care. A general
 -- guideline:
 --
 -- If you are allocating a resource that should be shared by multiple threads,
 -- and will be held for a long time, you should allocate it at the beginning of
--- a new @ResourceT@ block and then call @resourceForkIO@ from there.
+-- a new @ResourceT@ block and then call @resourceForkWith@ from there.
 --
--- Since 0.3.0
-resourceForkIO :: MonadBaseControl IO m => ResourceT m () -> ResourceT m ThreadId
-resourceForkIO (ResourceT f) = ResourceT $ \r -> L.mask $ \restore ->
+-- @since 1.1.9
+resourceForkWith :: MonadBaseControl IO m => (IO () -> IO a) -> ResourceT m () -> ResourceT m a
+resourceForkWith g (ResourceT f) = ResourceT $ \r -> L.mask $ \restore ->
     -- We need to make sure the counter is incremented before this call
     -- returns. Otherwise, the parent thread may call runResourceT before
     -- the child thread increments, and all resources will be freed
@@ -262,13 +266,19 @@ resourceForkIO (ResourceT f) = ResourceT $ \r -> L.mask $ \restore ->
         (stateAlloc r)
         (return ())
         (return ())
-        (liftBaseDiscard forkIO $ bracket_
+        (liftBaseDiscard g $ bracket_
             (return ())
             (stateCleanup ReleaseNormal r)
             (stateCleanup ReleaseException r)
             (restore $ f r))
 
-
+-- | Launch a new reference counted resource context using @forkIO@.
+--
+-- This is defined as @resourceForkWith forkIO@.
+--
+-- @since 0.3.0
+resourceForkIO :: MonadBaseControl IO m => ResourceT m () -> ResourceT m ThreadId
+resourceForkIO = resourceForkWith forkIO
 
 -- | A @Monad@ which can be used as a base for a @ResourceT@.
 --
