@@ -1,8 +1,11 @@
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE Safe #-}
 -- | This was previously known as the Resource monad. However, that term is
 -- confusing next to the ResourceT transformer, so it has been renamed.
 module Data.Acquire
     ( Acquire
     , with
+    , withEx
     , mkAcquire
     , mkAcquireType
     , allocateAcquire
