@@ -1,12 +1,15 @@
 {-# OPTIONS_HADDOCK not-home #-}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE Trustworthy #-}
 module Data.Acquire.Internal
     ( Acquire (..)
     , Allocated (..)
     , with
+    , withEx
     , mkAcquire
     , ReleaseType (..)
     , mkAcquireType
@@ -19,6 +22,8 @@ import Control.Monad.Trans.Control (MonadBaseControl, control)
 import qualified Control.Exception.Lifted as E
 import Data.Typeable (Typeable)
 import Control.Monad (liftM, ap)
+import qualified Control.Monad.Catch as C
+import GHC.IO (unsafeUnmask)
 
 -- | The way in which a release is called.
 --
@@ -48,11 +53,11 @@ newtype Acquire a = Acquire ((forall b. IO b -> IO b) -> IO (Allocated a))
 instance Functor Acquire where
     fmap = liftM
 instance Applicative Acquire where
-    pure = return
+    pure a = Acquire (\_ -> return (Allocated a (const $ return ())))
     (<*>) = ap
 
 instance Monad Acquire where
-    return a = Acquire (\_ -> return (Allocated a (const $ return ())))
+    return = pure
     Acquire f >>= g' = Acquire $ \restore -> do
         Allocated x free1 <- f restore
         let Acquire g = g' x
@@ -102,6 +107,33 @@ with :: MonadBaseControl IO m
      -> m b
 with (Acquire f) g = control $ \run -> E.mask $ \restore -> do
     Allocated x free <- f restore
-    res <- run (g x) `E.onException` free ReleaseException
+    res <- restore (run (g x)) `E.onException` free ReleaseException
     free ReleaseNormal
     return res
+
+-- | Same as @with@, but uses the @MonadMask@ typeclass from exceptions instead
+-- of @MonadBaseControl@ from exceptions.
+--
+-- Since 1.1.3
+#if MIN_VERSION_exceptions(0,6,0)
+withEx :: (C.MonadMask m, MonadIO m)
+#else
+withEx :: (C.MonadCatch m, MonadIO m)
+#endif
+       => Acquire a
+       -> (a -> m b)
+       -> m b
+withEx (Acquire f) g = do
+    -- We need to do some funny business, since the restore we get below is
+    -- specialized to the m from the result, whereas we need a restore function
+    -- in IO. Checking the current masking state is exactly how mask is
+    -- implemented in base.
+    origMS <- liftIO E.getMaskingState
+
+    C.mask $ \restore -> do
+        Allocated x free <- liftIO $ f $ case origMS of
+            E.Unmasked -> unsafeUnmask
+            _ -> id
+        res <- restore (g x) `C.onException` liftIO (free ReleaseException)
+        liftIO $ free ReleaseNormal
+        return res
